<?php

namespace Drupal\mailchimp_marketing\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ConnectException;
use MailchimpMarketing\ApiClient;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Provides Mailchimp controller.
 */
class MailchimpController extends ControllerBase {
  use StringTranslationTrait;

  /**
   * Config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected $emailValidator;

  /**
   * The Logger channel service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $loggerChannel;

  /**
   * Constructs a new CoffeeController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Component\Utility\EmailValidatorInterface $validator
   *   The email validator.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The Logger service.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger,
    ConfigFactoryInterface $config_factory,
    EmailValidatorInterface $validator
  ) {
    $this->loggerChannel = $logger->get('mailchimp_marketing');
    $this->config = $config_factory->get('mailchimp_marketing.settings');
    $this->emailValidator = $validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory"'),
      $container->get('config.factory'),
      $container->get('email.validator')
    );
  }

  /**
   * Mailchimp connection getter.
   *
   * @return \MailchimpMarketing\ApiClient
   *   Mailchimp connection object.
   */
  public function getConnection() {
    $api_key = $this->config->get('mailchimp_api_key');
    $server_prefix = $this->config->get('mailchimp_server_prefix');

    // Test the connection.
    $mailchimp = new ApiClient();
    $mailchimp->setConfig([
      'apiKey' => $api_key,
      'server' => $server_prefix,
    ]);

    return $mailchimp;
  }

  /**
   * Test connection.
   *
   * @return bool
   *   TRUE if connection successful.
   */
  public function pingSuccess() {
    try {
      $mailchimp = $this->getConnection();
      $response = $mailchimp->ping->get();
      return ($response->health_status == "Everything's Chimpy!");
    }
    catch (ClientException $e) {
      return FALSE;
    }
    catch (ConnectException $e) {
      return FALSE;
    }
  }

  /**
   * Test connection.
   *
   * @return bool
   *   TRUE if connection successful.
   */
  public function getDefaultList() {
    $mailchimp = $this->getConnection();
    return $this->config->get('mailchimp_default_list');
  }

  /**
   * Checks if user is member of the list by email
   *
   * @param string $email
   *   Value of email.
   * @param string $listId
   *   Value of Mailchimp list id.
   *
   * @return object|bool
   *   Member object or FALSE if not found.
   */
  public function getMember($email, $listId = NULL) {
    $mailchimp = $this->getConnection();

    if (!$this->emailValidator->isValid($email)) {
      $this->loggerChannel->notice("Invalid email address.");
      return FALSE;
    }

    if (!isset($listId)) {
      $listId = $this->config->get('mailchimp_default_list');
    }

    if (!isset($listId)) {
      $this->loggerChannel->warning("Mailchimp list is undefined.");
      return FALSE;
    }

    try {
      $memberId = md5($email);
      $response = $mailchimp->lists->getListMember($listId, $memberId);
    }
    catch (\Exception $exception) {
      $this->loggerChannel->error($exception->getMessage());
      return FALSE;
    }

    return $response;
  }

  /**
   * Checks if user is member of the list by unique_email_id
   *
   * @param string $uniqueId
   *   Value of unique_email_id.
   * @param string $listId
   *   Value of Mailchimp list id.
   *
   * @return object|bool
   *   Member object or FALSE if not found.
   */
  public function getMemberByUniqueId(string $uniqueId, $listId = NULL) {
    $mailchimp = $this->getConnection();

    if (empty($uniqueId)) {
      $this->loggerChannel->notice("Invalid unique email id.");
      return FALSE;
    }

    if (!isset($listId)) {
      $listId = $this->config->get('mailchimp_default_list');
    }

    if (!isset($listId)) {
      $this->loggerChannel->warning("Mailchimp list is undefined.");
      return FALSE;
    }

    try {
      $response = $mailchimp->lists->getListMembersInfo($listId, null, null, null, null, null, null, null, null, null, null, $uniqueId);
      if ($response->total_items > 0) {
        $response = $response->members[0];
      }
      else {
        return FALSE;
      }
    }
    catch (\Exception $exception) {
      $this->loggerChannel->error($exception->getMessage());
      return FALSE;
    }

    return $response;
  }

  /**
   * Add email to mailchimp list.
   *
   * @param string $email
   *   Value of email.
   * @param string $listId
   *   Value of Mailchimp list id.
   * @param array $options
   *   Other options for request, example:
   *    [
   *     "email_type" => "html",
   *     "tags": [],
   *    ].
   *    See https://mailchimp.com/developer/api/marketing/list-members/add-member-to-list/ for full list of parameters.
   *  @param string $memberStatus
   *    Status of member, 'subscribed' is default, but if double opt-in is required set it to 'pending'
   *
   * @return object|bool
   *   Member object or FALSE if failed to add a new subscriber.
   */
  public function addMember(string $email, string $listId = NULL, array $options = [], string $memberStatus = 'subscribed') {
    if (!isset($listId)) {
      $listId = $this->config->get('mailchimp_default_list');
    }

    if (!isset($listId)) {
      $this->loggerChannel->warning("Mailchimp list is undefined.");
      return FALSE;
    }

    $member = $this->getMember($email, $listId);

    if (!$member) {
      // Add new member.
      $request = [
        "email_address" => $email,
        "status" => $memberStatus,
      ];

      if (isset($options)) {
        $request = array_merge($request, $options);
      }

      try {
        $mailchimp = $this->getConnection();

        // To successfully add tags to new user, custom keys should be removed.
        if (isset($request['tags']) && is_array($request['tags'])) {
          $mailchimp_tags_no_ids = [];
          foreach ($request['tags'] as $mailchimp_tag) {
            $mailchimp_tags_no_ids[] = $mailchimp_tag;
          }
          $request['tags'] = $mailchimp_tags_no_ids;
        }

        $member = $mailchimp->lists->addListMember($listId, $request);
      }
      catch (\Exception $exception) {
        $this->loggerChannel->error($exception->getMessage());
        return FALSE;
      }
    }
    else {
      // Check user's status.
      if ($member->status == 'unsubscribed') {
        $this->loggerChannel->error($this->t('User has unsubscribed from this mail list and cannot be re-subscribed.'));
        return FALSE;
      }

      // Update existing subscriber.
      try {
        $mailchimp = $this->getConnection();
        if (isset($options) && isset($options['tags']) && count($options['tags']) > 0) {
          $requestTags = [];
          $mailchimpTags = $mailchimp->lists->getListMemberTags($listId, $member->id, NULL, NULL, 200);
          $tags = $options['tags'];

          // Set members tags from mailchimp to inactive.
          // @TODO: remove tags if options is empty.
          foreach ($mailchimpTags->tags as $tag) {
            if (!in_array($tag->name, $tags)) {
              $requestTags[] = [
                "name" => $tag->name,
                "status" => 'inactive',
              ];
            }
          }

          // Set new tags to active.
          foreach ($tags as $tag) {
            $requestTags[] = [
              "name" => $tag,
              "status" => "active",
            ];
          }

          $request = [
            "tags" => $requestTags,
          ];

          $mailchimp->lists->updateListMemberTags(
            $listId,
            $member->id,
            $request
          );
        }

        unset($options['tags']);
        if (!empty($options)) {
          $member = $mailchimp->lists->updateListMember(
            $listId,
            $member->id,
            $options
          );
        }
      }
      catch (\Exception $exception) {
        $this->loggerChannel->error($exception->getMessage());
        return FALSE;
      }
    }

    return isset($member) ? $member : FALSE;
  }
}
